package com.chelvc.framework.wechat.support;

import java.util.List;
import java.util.Map;
import java.util.Objects;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.wechat.PayChannel;
import com.chelvc.framework.wechat.WechatPayRequest;
import com.chelvc.framework.wechat.WechatPaymentCallback;
import com.chelvc.framework.wechat.WechatPaymentHandler;
import com.chelvc.framework.wechat.WechatUnifiedOrder;
import com.google.common.collect.Lists;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 微信支付处理器委派实现
 *
 * @author Woody
 * @date 2024/6/19
 */
@Slf4j
public class DelegatingWechatPaymentHandler implements WechatPaymentHandler {
    private final List<WechatPaymentHandler> handlers;

    public DelegatingWechatPaymentHandler(@NonNull List<WechatPaymentHandler> handlers) {
        AssertUtils.nonempty(handlers, () -> "Delegating handlers must not be empty");
        this.handlers = Lists.newArrayList(handlers);
    }

    /**
     * 查找微信支付处理器
     *
     * @param name 处理器名称
     * @return 处理器实例
     */
    private WechatPaymentHandler lookupPaymentHandler(String name) {
        WechatPaymentHandler optional = null;
        String merchant = ApplicationContextHolder.getProperty("wechat.payment." + name + ".merchant");
        for (WechatPaymentHandler handler : this.handlers) {
            if (handler.supports(name)) {
                if (optional == null) {
                    optional = handler;
                }

                // 优先使用指定商户号支付配置
                if (StringUtils.isEmpty(merchant) || handler.isMerchant(name, merchant)) {
                    return handler;
                }
            }
        }
        if (optional == null) {
            throw new IllegalArgumentException("Wechat payment handler not found: " + name);
        }
        return optional;
    }

    @Override
    public boolean supports(String name) {
        for (WechatPaymentHandler handler : this.handlers) {
            if (handler.supports(name)) {
                return true;
            }
        }
        return false;
    }

    @Override
    public boolean isChannel(String name, PayChannel channel) {
        for (WechatPaymentHandler handler : this.handlers) {
            if (handler.isChannel(name, channel)) {
                return true;
            }
        }
        return false;
    }

    @Override
    public boolean isMerchant(String name, String merchant) {
        for (WechatPaymentHandler handler : this.handlers) {
            if (handler.isMerchant(name, merchant)) {
                return true;
            }
        }
        return false;
    }

    @Override
    public String sign(@NonNull String name, @NonNull Map<String, String> parameters) {
        return this.lookupPaymentHandler(name).sign(name, parameters);
    }

    @Override
    public WechatPaymentCallback callback(@NonNull String name, @NonNull String content) {
        for (WechatPaymentHandler handler : this.handlers) {
            if (handler.supports(name)) {
                try {
                    WechatPaymentCallback callback = handler.callback(name, content);
                    if (Objects.nonNull(callback)) {
                        return callback;
                    }
                } catch (Exception e) {
                    log.error("Wechat payment callback handle failed: {}", content, e);
                }
            }
        }
        return null;
    }

    @Override
    public WechatUnifiedOrder unifiedorder(@NonNull String name, @NonNull WechatPayRequest request) {
        return this.lookupPaymentHandler(name).unifiedorder(name, request);
    }
}
